--
--
-- CPLD Tain VHDL Module - Train Control State Machine for Altera University Board 
-- Uses Flex 10K20 Device
--
-- Jim Hamblen, Georgia Tech School of ECE
--
-- VGA output displays train and switch state
-- Students supply train_control state machine (tcontrol.vhd) to control train
-- PB1 is run/stop 
-- PB2 is reset 
-- Sw0..3 is trainB speed
-- Sw7..4 is trainA speed
-- SensorX are track sensors for train (inputs for state machine)
-- SwitchX are track switches
-- TrackX select power source A or B for track segment
-- DirX selects direction: 00-stop  01-clockwise  10-counterclockwise
-- Note: Screen blinks when trains crash or go through switch in wrong direction
--
--                -----------Sw3--------------
--                | T1         \T4        T1 |
--                |     -------|-------      |
--                |     | T3   |   T3 |      |
--                |     |      | S5   |      |
--             S1 |   S2|             | S3   | S4
--                |     \     T2      /      |
--                ------Sw1---------Sw2-------
--
--                       Track Layout
--
library IEEE;
use  IEEE.STD_LOGIC_1164.all;
use  IEEE.STD_LOGIC_ARITH.all;
use  IEEE.STD_LOGIC_UNSIGNED.all;
LIBRARY lpm;
USE lpm.lpm_components.ALL;


entity train is

Generic(ADDR_WIDTH: integer := 12; DATA_WIDTH: integer := 1);

   port( signal PB1, PB2, Clock : in std_logic;
        signal LSB_a, LSB_b, LSB_c, LSB_d, LSB_e, LSB_f, LSB_g, LSB_dp,
        MSB_a, MSB_b, MSB_c, MSB_d, MSB_e, MSB_f, MSB_g, MSB_dp : out std_logic;               
        signal Red,Green,Blue : out std_logic;
        signal Horiz_sync,Vert_sync : out std_logic;
        signal MOUSE_DATA: in std_logic;
        signal MOUSE_CLK : in std_logic;
        signal Flex_Switch_1, Flex_Switch_2, Flex_Switch_3, Flex_Switch_4: in std_logic;
        signal Flex_Switch_5, Flex_Switch_6, Flex_Switch_7, Flex_Switch_8: in std_logic;
	-- Output All Train control state machine I/Os
		signal o_sensor1, o_sensor2, o_sensor3, o_sensor4, o_sensor5 : out std_logic;
		signal o_switch1, o_switch2, o_switch3 :  out std_logic;
		signal o_dirA1, o_dirA0, o_dirB1, o_dirB0:  out std_logic;
		signal o_track1, o_track2, o_track3, o_track4 : out std_logic;
        signal o_clock, o_reset : out std_logic);
  
		
end train;

architecture behavior of train is

component Tcontrol
port(reset, clock, sensor1, sensor2, sensor3, sensor4, sensor5: in std_logic;
     switch1, switch2, switch3: out std_logic;
     track1, track2, track3, track4: out std_logic;
     dirA, dirB : out std_logic_vector(1 Downto 0));
end component;

signal speed_A, speed_B: std_logic_vector(3 Downto 0);
signal switch, switch_sync, cycle_count: std_logic_vector(7 Downto 0);
signal speed_signal_A, speed_signal_B, train_run, train_stop, reset: std_logic;


-- Train Signals for virtual train display
signal trainArow, trainAcol: std_logic_vector(6 Downto 0);
signal next_trainArow, next_trainAcol: std_logic_vector(6 Downto 0);
signal next_trainBrow, next_trainBcol: std_logic_vector(6 Downto 0);
signal old_trainArow, old_trainAcol: std_logic_vector(6 Downto 0);
signal old2_trainArow, old2_trainAcol: std_logic_vector(6 Downto 0);
signal trainBrow, trainBcol: std_logic_vector(6 Downto 0);
signal old_trainBrow, old_trainBcol: std_logic_vector(6 Downto 0);
signal old2_trainBrow, old2_trainBcol: std_logic_vector(6 Downto 0);
signal col_adjustA, row_adjustA: std_logic_vector(6 Downto 0);
signal col_adjustB, row_adjustB: std_logic_vector(6 Downto 0);
signal C10A,C30A,C40A,C50A,C70A,R10A,R30A,R70A,R4xA,R5xA,T1,T2,T3,T4: boolean;
signal C10B,C30B,C40B,C50B,C70B,R10B,R30B,R70B,R4xB,R5xB: boolean;
signal crash, crashA, crashB, second: std_logic;
signal sensor, old_sensor: std_logic_vector(4 Downto 0);

-- Train Signals for state machine control
signal sensor1, sensor2, sensor3, sensor4, sensor5 : std_logic;
signal switch1, switch2, switch3 :  std_logic;
signal dirA, dirB:  std_logic_vector(1 Downto 0);
signal track1, track2, track3, track4 : std_logic;

-- Video Display Signals   
signal rom_address: std_logic_vector(11 Downto 0);
signal H_count,V_count: std_logic_vector(9 Downto 0);
signal F_count: std_logic_vector(4 Downto 0);
signal Color_count: std_logic_vector(3 Downto 0);
signal data: std_logic_vector(0 Downto 0);
signal Red_Data, Green_Data, Blue_Data, we, slow_clock: std_logic;
signal Red_Mux_Data, Green_Mux_Data, Blue_Mux_Data: std_logic;
signal Red_Display_Data, Green_Display_Data, Blue_Display_Data, display_item, Power_on: std_logic;

-- Signals for Video Memory for Pixel Data
signal address: std_logic_vector(13 Downto 0);
signal col_address, row_address: std_logic_vector(6 Downto 0);
signal pixel_col_count, pixel_row_count: std_logic_vector(5 Downto 0);
signal rom_output: std_logic_vector(0 Downto 0);

-- Signals for LED Display
signal LSB,MSB: std_logic_vector(3 Downto 0);
signal LSB_7SEG,MSB_7SEG: std_logic_vector(6 Downto 0);

-- Signals for Push buttons
signal PB1_sync, PB2_sync, PB2_Single_Pulse, PB1_Single_Pulse: std_logic; 
signal PB2_debounced, PB1_debounced, PB2_debounced_Sync, PB1_debounced_Sync: std_logic; 
signal PB1_debounced_delay, PB2_debounced_delay, Debounce_clock: std_logic;
signal SHIFT_PB1, SHIFT_PB2: std_logic_vector(3 Downto 0);

constant H_max : std_logic_vector(9 Downto 0) := CONV_STD_LOGIC_VECTOR(799,10); 
-- 799 is max horiz count
constant V_max : std_logic_vector(9 Downto 0) := CONV_STD_LOGIC_VECTOR(524,10); 
-- 524 is max vert count
signal video_on, video_on_H, video_on_V: std_logic;

begin           
------------------------------------------------------------------------------
-- 64 by 64 by 1 video rom for pixel background data
--
back_rom: lpm_rom

      GENERIC MAP (lpm_widthad => ADDR_WIDTH,
        lpm_outdata => "UNREGISTERED",
        lpm_address_control => "UNREGISTERED",
        lpm_file => "train.mif",
         lpm_width => DATA_WIDTH)
-- Reads in mif file for initial data - train track background
     
PORT MAP (address => rom_address(11 downto 0), q => rom_output);

CONTROL: Tcontrol
-- Use Student supplied module for Train control
 port map( reset => reset, clock => slow_clock,
           sensor1 => sensor1, sensor2 => sensor2,
           sensor3 => sensor3, sensor4 => sensor4,
           sensor5 => sensor5,
           switch1 => switch1, switch2 => switch2,
           switch3 => switch3,
           track1 => track1, track2 => track2,
           track3 => track3, track4 => track4,
           dirA => dirA, dirB => dirB);


-- Copy state machine I/Os to FLEX chip output pins for use by logic analyzer
o_sensor1 <= sensor1;     
o_sensor2 <= sensor2;     
o_sensor3 <= sensor3;     
o_sensor4 <= sensor4;     
o_sensor5 <= sensor5;     
o_switch1 <= switch1;
o_switch2 <= switch2;
o_switch3 <= switch3;
o_track1 <= track1;		
o_track2 <= track2;		
o_track3 <= track3;		
o_track4 <= track4;
o_dirA1 <= dirA(1);
o_dirA0 <= dirA(0);
o_dirB1 <= dirB(1);
o_dirB0 <= dirB(0);
o_clock <= slow_clock;
o_reset <= reset;

		
-- Colors for pixel data on video signal
-- address video_ram for pixel background color data
-- If display_item='1' use display data instead of background
Red_Mux_Data <= rom_output(0) When display_item='0' else red_display_data;
Green_Mux_Data <= rom_output(0) When display_item='0' else green_display_data;
Blue_Mux_Data <= '1' When display_item='0' else blue_display_data;

Red_Data <=  (Red_Mux_Data xor (Crash and Second));
Green_Data <= (Green_Mux_Data xor (Crash and Second));
Blue_Data <= Blue_Mux_Data;

rom_address(11 Downto 6) <= row_address(6 downto 1);
rom_address(5 Downto 0) <= col_address(6 downto 1);

reset <= PB2_Debounced_SYNC;
slow_clock <= Debounce_Clock;


Red <=   Red_Data and video_on;
Green <= Green_Data and video_on;
Blue <=  Blue_Data and video_on;

-- video_on turns off pixel data when not in the view area
video_on <= video_on_H and video_on_V;
Power_on <='0';

-- Combine Flex Dip Switch Inputs into Switch vector
Switch <= Flex_Switch_8 & Flex_Switch_7 & Flex_Switch_6 & Flex_Switch_5 &
          Flex_Switch_4 & Flex_Switch_3 & Flex_Switch_2 & Flex_Switch_1;

-- Controls Speed of TrainA with input from Flex Switches 7..4
SPEED_TRAINA: process 
begin
  wait until (slow_clock'Event) and (slow_clock='1');
if Speed_A = Switch_Sync(7 Downto 4) then
 Speed_A <="0000"; 
 Speed_Signal_A <='1';
ELSE
 Speed_A <= Speed_A + 1;
 Speed_Signal_A <='0';
End if;
end process SPEED_TRAINA;

SPEED_TRAINB: process 
begin
  wait until (slow_clock'Event) and (slow_clock='1');
if Speed_B = Switch_Sync(3 Downto 0) then
 Speed_B <="0000"; 
 Speed_Signal_B <='1';
ELSE
 Speed_B <= Speed_B + 1;
 Speed_Signal_B <= '0';
End if;
end process SPEED_TRAINB;

-- Makes PB1 the Train Run/Step Button
PAUSE_TRAIN: process (PB1_DEBOUNCED_SYNC)
begin
  wait until (PB1_DEBOUNCED_SYNC'Event) and (PB1_DEBOUNCED_SYNC='1');
    Train_Run <= Not Train_Run;
    Old_sensor <= Sensor;
End process PAUSE_TRAIN;

Sensor <= Sensor5 & Sensor4 & Sensor3 & Sensor2 & Sensor1;

-- Stops Train only at Sensor Change for Single Step
STEP_TRAIN: process (slow_clock)
begin
  wait until (slow_clock'event) and (slow_clock = '1');
   IF Train_run = '1' Then Train_stop <= '1'; Else
   IF (Sensor /= Old_Sensor) Then 
    Train_stop <= '0';
   End if;
   End if;
End process STEP_TRAIN;

-- Move Train
C10A <= (trainAcol(6 Downto 1) = "001000");
C30A <= (trainAcol(6 Downto 1) = "011000");
C40A <= (trainAcol(6 Downto 1)= "100000");
C50A <= (trainAcol(6 Downto 1) = "101000");
C70A <= (trainAcol(6 Downto 1) = "111000");
R10A <= (trainArow(6 Downto 1) = "001000");
R30A <= (trainArow(6 Downto 1) = "011000");
R4xA <= (trainArow(6 Downto 4) = "100");
R5xA <= (trainArow(6 Downto 4) = "101");
R70A <= (trainArow(6 Downto 1) = "111000");

C10B <= (trainBcol(6 Downto 1) = "001000");
C30B <= (trainBcol(6 Downto 1) = "011000");
C40B <= (trainBcol(6 Downto 1)= "100000");
C50B <= (trainBcol(6 Downto 1) = "101000");
C70B <= (trainBcol(6 Downto 1) = "111000");
R10B <= (trainBrow(6 Downto 1) = "001000");
R30B <= (trainBrow(6 Downto 1) = "011000");
R4xB <= (trainBrow(6 Downto 4) = "100");
R5xB <= (trainBrow(6 Downto 4) = "101");
R70B <= (trainBrow(6 Downto 1) = "111000");

T1 <= (track1 = '1');
T2 <= (track2 = '1');
T3 <= (track3 = '1');
T4 <= (track4 = '1');

Next_TrainAcol <= TrainAcol + col_adjustA;
Next_TrainArow <= TrainArow + row_adjustA;
Next_TrainBcol <= TrainBcol + col_adjustB;
Next_TrainBrow <= TrainBrow + row_adjustB;
Crash <= CrashA or CrashB;

MOVE_TRAINA: Process
Begin
 
   wait until (slow_clock'Event) and (slow_clock = '1');
IF reset='1' or trainArow = "0000000" Then
 trainArow <= "0010000";
 trainAcol <= "0100000";
 old_trainArow <= "0010000";
 old_trainAcol <= "0100010";
 col_adjustA <= "0000000";
 row_adjustA <= "0000000";
 crashA <='0';
Else

If (trainArow=trainBrow) and (trainAcol=trainBcol) then 
 crashA <='1';
end if;

If (Speed_Signal_A = '1') and (crash='0')  and (Train_stop = '1') Then

Old2_TrainAcol <= Old_TrainAcol;
Old2_TrainArow <= Old_TrainArow;
Old_TrainAcol <= TrainAcol;
Old_TrainArow <= TrainArow;
TrainAcol <= Next_TrainAcol;
TrainArow <= Next_TrainArow;
-----------------------------------------------------------
-- Track 1
IF R10A and not(C10A) and not(C70A) and not(C40A) Then
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustA <= "1111111";
  row_adjustA <= "0000000";
 Else
-- counter clockwise
  IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
   col_adjustA <= "0000001";
   row_adjustA <= "0000000";
  Else
   col_adjustA <= "0000000";
   row_adjustA <= "0000000";
  End if;
 End if;
Else

IF R10A and C10A Then
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "0000001";
 Else
-- counter clockwise
 IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
  col_adjustA <= "0000001";
  row_adjustA <= "0000000";
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
Else

-- Switch 3 Location
IF R10A and C40A Then
 IF Switch3 = '0' Then
-- switch closed
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustA <= "1111111";
  row_adjustA <= "0000000";
-- crash if going wrong way into switch
  IF old_trainArow > "0010000" Then crashA <= '1'; End if;
 Else
-- counter clockwise
 IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
  col_adjustA <= "0000001";
  row_adjustA <= "0000000";
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
Else
-- switch open
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustA <= "1111111";
  row_adjustA <= "0000000";
-- crash if going wrong way into switch
  IF old_trainAcol > "1000001" Then crashA <= '1'; End if;
 Else
-- counter clockwise
 IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "0000001";
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
 End if;
Else

IF R10A and C70A Then
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustA <= "1111111";
  row_adjustA <= "0000000";
 Else
-- counter clockwise
 IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "0000001";
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
Else

IF C10A and not(R10A) and not(R70A) Then
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "0000001";
 Else
-- counter clockwise
 IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "1111111";
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
Else

IF C10A and R70A Then
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustA <= "0000001";
  row_adjustA <= "0000000";
 Else
-- counter clockwise
 IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "1111111";
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
Else

IF C70A and not(R70A) and not(R10A) Then
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "1111111";
 Else
-- counter clockwise
 IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "0000001";
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
Else

-- Track 2 When col>=30 and <=50 else Track 1
IF R70A and not(C10A) and not(C70A) and not(C30A) and not(C50A) Then
 IF trainAcol >= "0110000" and trainAcol <= "1010001" Then
-- Track 2 region
-- clockwise
 IF (not(T2) and DIRA(0)='1') or (T2 and DIRB(0)='1') Then
  col_adjustA <= "0000001";
  row_adjustA <= "0000000";
 Else
-- counter clockwise
 IF (not(T2) and DIRA(1)='1') or (T2 and DIRB(1)='1') Then
  col_adjustA <= "1111111";
  row_adjustA <= "0000000";
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
Else

-- Track 1 region
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustA <= "0000001";
  row_adjustA <= "0000000";
 Else
-- counter clockwise
 IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
  col_adjustA <= "11111111";
  row_adjustA <= "0000000";
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
End if;
Else

IF R70A and C70A Then
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "1111111";
 Else
-- counter clockwise
 IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
  col_adjustA <= "1111111";
  row_adjustA <= "0000000";
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
Else

-- Switch 1 Location
IF R70A and C30A Then
 IF Switch1 = '0' Then
-- switch closed
-- clockwise
 IF (not(T2) and DIRA(0)='1') or (T2 and DIRB(0)='1') Then
  col_adjustA <= "0000001";
  row_adjustA <= "0000000";
-- crash if going wrong way into switch
  IF old_trainArow < "1110000" Then crashA <= '1'; End if;
 Else
-- counter clockwise
 IF (not(T2) and DIRA(1)='1') or (T2 and DIRB(1)='1') Then
  col_adjustA <= "1111111";
  row_adjustA <= "0000000";
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
Else
-- switch open
-- clockwise
 IF (not(T2) and DIRA(0)='1') or (T2 and DIRB(0)='1') Then
  col_adjustA <= "0000001";
  row_adjustA <= "0000000";
 Else
-- counterclockwise
 IF (not(T2) and DIRA(1)='1') or (T2 and DIRB(1)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "1111111";
-- crash if going wrong way into switch
  IF old_trainAcol < "0110000" Then crashA <= '1'; End if;
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
 End if;
Else

-- Switch 2 Location
IF R70A and C50A Then
 IF Switch2 = '0' Then
-- switch closed
-- clockwise
 IF (not(T2) and DIRA(0)='1') or (T2 and DIRB(0)='1') Then
  col_adjustA <= "0000001";
  row_adjustA <= "0000000";
 Else
-- counter clockwise
 IF (not(T2) and DIRA(1)='1') or (T2 and DIRB(1)='1') Then
  col_adjustA <= "1111111";
  row_adjustA <= "0000000";
-- crash if going wrong way into switch
  IF old_trainArow < "1110000" Then crashA <= '1'; End if;
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
Else
-- switch open
-- clockwise
 IF (not(T2) and DIRA(0)='1') or (T2 and DIRB(0)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "1111111";
 Else
-- counter clockwise
 IF (not(T2) and DIRA(1)='1') or (T2 and DIRB(1)='1') Then
  col_adjustA <= "1111111";
  row_adjustA <= "0000000";
-- crash if going wrong way into switch
  IF old_trainAcol > "1010000" Then crashA <= '1'; End if;
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
 End if;
Else

------------------------------------------------------------
-- Track 3
IF C30A and not(R70A) and not(R30A) Then
-- clockwise
 IF (not(T3) and DIRA(0)='1') or (T3 and DIRB(0)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "0000001";
 Else
-- counter clockwise
 IF (not(T3) and DIRA(1)='1') or (T3 and DIRB(1)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "1111111";
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
Else

IF C30A and R30A Then
-- clockwise
 IF (not(T3) and DIRA(0)='1') or (T3 and DIRB(0)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "0000001";
 Else
--  counter clockwise
 IF (not(T3) and DIRA(1)='1') or (T3 and DIRB(1)='1') Then
  col_adjustA <= "0000001";
  row_adjustA <= "0000000";
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
Else

IF R30A and not(C50A) and not(C30A) and not(C40A) Then
-- clockwise
 IF (not(T3) and DIRA(0)='1') or (T3 and DIRB(0)='1') Then
  col_adjustA <= "1111111";
  row_adjustA <= "0000000";
 Else
--  counter clockwise
 IF (not(T3) and DIRA(1)='1') or (T3 and DIRB(1)='1') Then
  col_adjustA <= "0000001";
  row_adjustA <= "0000000";
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
Else
 
IF R30A and C40A Then
-- Tracks cross which way was train moving?
IF Old_TrainArow /= "0110000" or TrainArow(0)='1' Then
-- N/S Track
-- clockwise
 IF (not(T4) and DIRA(0)='1') or (T4 and DIRB(0)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "1111111";
 Else
-- counter clockwise
 IF (not(T4) and DIRA(1)='1') or (T4 and DIRB(1)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "0000001";
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
Else
-- E/W Track
-- clockwise
 IF (not(T3) and DIRA(0)='1') or (T3 and DIRB(0)='1') Then
  col_adjustA <= "1111111";
  row_adjustA <= "0000000";
 Else
--  counter clockwise
 IF (not(T3) and DIRA(1)='1') or (T3 and DIRB(1)='1') Then
  col_adjustA <= "0000001";
  row_adjustA <= "0000000";
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
End if;
Else


IF C50A and not(R70A) and not(R30A) Then
-- clockwise
 IF (not(T3) and DIRA(0)='1') or (T3 and DIRB(0)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "1111111";
 Else
-- counter clockwise
 IF (not(T3) and DIRA(1)='1') or (T3 and DIRB(1)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "0000001";
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
Else

IF C50A and R30A Then
-- clockwise
 IF (not(T3) and DIRA(0)='1') or (T3 and DIRB(0)='1') Then
  col_adjustA <= "1111111";
  row_adjustA <= "0000000";
 Else
-- counter clockwise
 IF (not(T3) and DIRA(1)='1') or (T3 and DIRB(1)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "0000001";
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
Else

--------------------------------------------------------------------------
-- Track 4
IF C40A and not(R10A) Then
-- clockwise
 IF (not(T4) and DIRA(0)='1') or (T4 and DIRB(0)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "1111111";
 Else
-- counter clockwise
 IF (not(T4) and DIRA(1)='1') or (T4 and DIRB(1)='1') Then
  col_adjustA <= "0000000";
  row_adjustA <= "0000001";
  IF trainArow > "1010000" Then CrashA <= '1'; End if;
 Else
  col_adjustA <= "0000000";
  row_adjustA <= "0000000";
 End if;
 End if;
Else
 col_adjustA <= "0000000";
 row_adjustA <= "0000000";
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;

End Process MOVE_TRAINA;


-----------------------------------------------------------
-- Move Train B
MOVE_TRAINB: Process
Begin
 
   wait until (slow_clock'Event) and (slow_clock = '1') ;
IF reset='1' or trainBrow = "0000000" Then
 trainbrow <= "1000000";
 trainbcol <= "0110000";
 old_trainbrow <= "1000010";
 old_trainbcol <= "0110000";
 col_adjustB <= "0000000";
 row_adjustB <= "0000000";
 crashB <= '0';
Else
If (trainArow=trainBrow) and (trainAcol=trainBcol) then 
 crashB <='1';
end if;

If (Speed_Signal_B = '1') and (crash='0') and (Train_stop ='1') Then

Old2_TrainBcol <= Old_TrainBcol;
Old2_TrainBrow <= Old_TrainBrow;
Old_TrainBcol <= TrainBcol;
Old_TrainBrow <= TrainBrow;
TrainBcol <= Next_TrainBcol;
TrainBrow <= Next_TrainBrow;
-----------------------------------------------------------
-- Track 1
IF R10B and not(C10B) and not(C70B) and not(C40B) Then
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustB <= "1111111";
  row_adjustB <= "0000000";
 Else
-- counter clockwise
  IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
   col_adjustB <= "0000001";
   row_adjustB <= "0000000";
  Else
   col_adjustB <= "0000000";
   row_adjustB <= "0000000";
  End if;
 End if;
Else

IF R10B and C10B Then
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "0000001";
 Else
-- counter clockwise
 IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
  col_adjustB <= "0000001";
  row_adjustB <= "0000000";
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
Else

-- Switch 3 Location
IF R10B and C40B Then
 IF Switch3 = '0' Then
-- switch closed
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustB <= "1111111";
  row_adjustB <= "0000000";
-- crash if going wrong way into switch
  IF old_trainBrow > "0010000" Then crashB <= '1'; End if;
 Else
-- counter clockwise
 IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
  col_adjustB <= "0000001";
  row_adjustB <= "0000000";
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
Else
-- switch open
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustB <= "1111111";
  row_adjustB <= "0000000";
-- crash if going wrong way into switch
  IF old_trainBcol > "1000000" Then crashB <= '1'; End if;
 Else
-- counter clockwise
 IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "0000001";
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
 End if;
Else

IF R10B and C70B Then
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustB <= "1111111";
  row_adjustB <= "0000000";
 Else
-- counter clockwise
 IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "0000001";
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
Else

IF C10B and not(R10B) and not(R70B) Then
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "0000001";
 Else
-- counter clockwise
 IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "1111111";
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
Else

IF C10B and R70B Then
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustB <= "0000001";
  row_adjustB <= "0000000";
 Else
-- counter clockwise
 IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "1111111";
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
Else

IF C70B and not(R70B) and not(R10B) Then
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "1111111";
 Else
-- counter clockwise
 IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "0000001";
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
Else

-- Track 2 When col>=30 and <=50 else Track 1
IF R70B and not(C10B) and not(C70B) and not(C30B) and not(C50B) Then
 IF trainBcol >= "0110000" and trainBcol <= "1010001" Then
-- Track 2 region
-- clockwise
 IF (not(T2) and DIRA(0)='1') or (T2 and DIRB(0)='1') Then
  col_adjustB <= "0000001";
  row_adjustB <= "0000000";
 Else
-- counter clockwise
 IF (not(T2) and DIRA(1)='1') or (T2 and DIRB(1)='1') Then
  col_adjustB <= "1111111";
  row_adjustB <= "0000000";
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
Else

-- Track 1 region
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustB <= "0000001";
  row_adjustB <= "0000000";
 Else
-- counter clockwise
 IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
  col_adjustB <= "11111111";
  row_adjustB <= "0000000";
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
End if;
Else

IF R70B and C70B Then
-- clockwise
 IF (not(T1) and DIRA(0)='1') or (T1 and DIRB(0)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "1111111";
 Else
-- counter clockwise
 IF (not(T1) and DIRA(1)='1') or (T1 and DIRB(1)='1') Then
  col_adjustB <= "1111111";
  row_adjustB <= "0000000";
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
Else

-- Switch 1 Location
IF R70B and C30B Then
 IF Switch1 = '0' Then
-- switch closed
-- clockwise
 IF (not(T2) and DIRA(0)='1') or (T2 and DIRB(0)='1') Then
  col_adjustB <= "0000001";
  row_adjustB <= "0000000";
-- crash if going wrong way into switch
  IF old_trainBrow < "1110000" Then crashB <= '1'; End if;
 Else
-- counter clockwise
 IF (not(T2) and DIRA(1)='1') or (T2 and DIRB(1)='1') Then
  col_adjustB <= "1111111";
  row_adjustB <= "0000000";
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
Else
-- switch open
-- clockwise
 IF (not(T2) and DIRA(0)='1') or (T2 and DIRB(0)='1') Then
  col_adjustB <= "0000001";
  row_adjustB <= "0000000";
 Else
-- counterclockwise
 IF (not(T2) and DIRA(1)='1') or (T2 and DIRB(1)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "1111111";
-- crash if going wrong way into switch
  IF old_trainBcol < "0110000" Then crashB <= '1'; End if;
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
 End if;
Else

-- Switch 2 Location
IF R70B and C50B Then
 IF Switch2 = '0' Then
-- switch closed
-- clockwise
 IF (not(T2) and DIRA(0)='1') or (T2 and DIRB(0)='1') Then
  col_adjustB <= "0000001";
  row_adjustB <= "0000000";
 Else
-- counter clockwise
 IF (not(T2) and DIRA(1)='1') or (T2 and DIRB(1)='1') Then
  col_adjustB <= "1111111";
  row_adjustB <= "0000000";
 -- crash if going wrong way into switch
  IF old_trainBrow < "1110000" Then crashB <= '1'; End if;
Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
Else
-- switch open
-- clockwise
 IF (not(T2) and DIRA(0)='1') or (T2 and DIRB(0)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "1111111";
 Else
-- counter clockwise
 IF (not(T2) and DIRA(1)='1') or (T2 and DIRB(1)='1') Then
  col_adjustB <= "1111111";
  row_adjustB <= "0000000";
-- crash if going wrong way into switch
  IF old_trainBcol > "1010000" Then crashB <= '1'; End if;
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
 End if;
Else

------------------------------------------------------------
-- Track 3
IF C30B and not(R70B) and not(R30B) Then
-- clockwise
 IF (not(T3) and DIRA(0)='1') or (T3 and DIRB(0)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "0000001";
 Else
-- counter clockwise
 IF (not(T3) and DIRA(1)='1') or (T3 and DIRB(1)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "1111111";
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
Else

IF C30B and R30B Then
-- clockwise
 IF (not(T3) and DIRA(0)='1') or (T3 and DIRB(0)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "0000001";
 Else
--  counter clockwise
 IF (not(T3) and DIRA(1)='1') or (T3 and DIRB(1)='1') Then
  col_adjustB <= "0000001";
  row_adjustB <= "0000000";
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
Else

IF R30B and not(C50B) and not(C30B) and not(C40B) Then
-- clockwise
 IF (not(T3) and DIRA(0)='1') or (T3 and DIRB(0)='1') Then
  col_adjustB <= "1111111";
  row_adjustB <= "0000000";
 Else
--  counter clockwise
 IF (not(T3) and DIRA(1)='1') or (T3 and DIRB(1)='1') Then
  col_adjustB <= "0000001";
  row_adjustB <= "0000000";
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
Else
 
IF R30B and C40B Then
-- Tracks cross which way was train moving?
IF Old_TrainBrow /= "0110000" or TrainBrow(0)='1'  Then
-- N/S Track
-- clockwise
 IF (not(T4) and DIRA(0)='1') or (T4 and DIRB(0)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "1111111";
 Else
-- counter clockwise
 IF (not(T4) and DIRA(1)='1') or (T4 and DIRB(1)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "0000001";
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
Else
-- E/W Track
-- clockwise
 IF (not(T3) and DIRA(0)='1') or (T3 and DIRB(0)='1') Then
  col_adjustB <= "1111111";
  row_adjustB <= "0000000";
 Else
--  counter clockwise
 IF (not(T3) and DIRA(1)='1') or (T3 and DIRB(1)='1') Then
  col_adjustB <= "0000001";
  row_adjustB <= "0000000";
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
End if;
Else


IF C50B and not(R70B) and not(R30B) Then
-- clockwise
 IF (not(T3) and DIRA(0)='1') or (T3 and DIRB(0)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "1111111";
 Else
-- counter clockwise
 IF (not(T3) and DIRA(1)='1') or (T3 and DIRB(1)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "0000001";
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
Else

IF C50B and R30B Then
-- clockwise
 IF (not(T3) and DIRA(0)='1') or (T3 and DIRB(0)='1') Then
  col_adjustB <= "1111111";
  row_adjustB <= "0000000";
 Else
-- counter clockwise
 IF (not(T3) and DIRA(1)='1') or (T3 and DIRB(1)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "0000001";
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
Else

--------------------------------------------------------------------------
-- Track 4
IF C40B and not(R10B) Then
-- clockwise
 IF (not(T4) and DIRA(0)='1') or (T4 and DIRB(0)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "1111111";
 Else
-- counter clockwise
 IF (not(T4) and DIRA(1)='1') or (T4 and DIRB(1)='1') Then
  col_adjustB <= "0000000";
  row_adjustB <= "0000001";
-- Check for end of Track 4
  IF trainBrow > "1010000" Then CrashB <= '1'; End if;
 Else
  col_adjustB <= "0000000";
  row_adjustB <= "0000000";
 End if;
 End if;
Else
 col_adjustB <= "0000000";
 row_adjustB <= "0000000";
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;
End if;

End Process MOVE_TRAINB;

-- Compute Track Sensor Values
COMPUTE_SENSOR: Process
Begin

 IF (C10A and R5xA) OR  (C10B and R5xB) Then
  sensor1 <= '1';
 Else
  sensor1 <= '0';
 End if;
 
 IF (C30A and R5xA) OR (C30B and R5xB) Then
  sensor2 <= '1';
 Else
  sensor2 <= '0';
 End if;

 IF (C50A and R5xA) OR (C50B and R5xB) Then
  sensor3 <= '1';
 Else
  sensor3 <= '0';
 End if;

 IF (C70A and R5xA) OR (C70B and R5xB) Then
  sensor4 <= '1';
 Else
  sensor4 <= '0';
 End if;

 IF (R4xA and C40A) OR (R4xB and C40B) Then
  sensor5 <= '1';
 Else
  sensor5 <= '0';
 End if;

End Process COMPUTE_SENSOR;

-- Display Sensor, Switch, and Train
TRAIN_DISPLAY: Process
Begin
Wait until(Clock'Event) and (Clock='1');

-- Display Train
-- Train A
If ((trainAcol(6 Downto 1) = col_address(6 Downto 1)) and 
   (trainArow(6 Downto 1) = row_address(6 Downto 1))) Then
     display_item <= '1';
     red_display_data <= '0';
     green_display_data <='0';
     blue_display_data <= '0';
else


If ((old_trainAcol(6 Downto 1) = col_address(6 Downto 1)) and 
   (old_trainArow(6 Downto 1) = row_address(6 Downto 1))) Then
     display_item <= '1';
     red_display_data <= '0';
     green_display_data <='0';
     blue_display_data <= '0';
else

If ((old2_trainAcol(6 Downto 1) = col_address(6 Downto 1)) and 
   (old2_trainArow(6 Downto 1) = row_address(6 Downto 1))) Then
     display_item <= '1';
     red_display_data <= '0';
     green_display_data <='0';
     blue_display_data <= '0';
else

-- Train B
If ((trainBcol(6 Downto 1) = col_address(6 Downto 1)) and 
   (trainBrow(6 Downto 1) = row_address(6 Downto 1))) Then
     display_item <= '1';
     red_display_data <= '1';
     green_display_data <='0';
     blue_display_data <= '0';
else

If ((old_trainBcol(6 Downto 1) = col_address(6 Downto 1)) and 
   (old_trainBrow(6 Downto 1) = row_address(6 Downto 1))) Then
     display_item <= '1';
     red_display_data <= '1';
     green_display_data <='0';
     blue_display_data <= '0';
else

If ((old2_trainBcol(6 Downto 1) = col_address(6 Downto 1)) and 
   (old2_trainBrow(6 Downto 1) = row_address(6 Downto 1))) Then
     display_item <= '1';
     red_display_data <= '1';
     green_display_data <='0';
     blue_display_data <= '0';
else



-- Display Switch
-- switch 3 display
If row_address(6 Downto 1) = "000110" and
   col_address(6 Downto 1) = "100000" Then
      display_item <= '1';
  If switch3 = '0' Then
     red_display_data <= '0';
     green_display_data <='1';
     blue_display_data <= '0';
    else
     red_display_data <= '1';
     green_display_data <='0';
     blue_display_data <= '0';
    end if;
else

-- switch 2 display
If row_address(6 Downto 1) = "111010" and
   col_address(6 Downto 1) = "101000" Then
     display_item <= '1';
    If switch2 = '0' Then
     red_display_data <= '0';
     green_display_data <='1';
     blue_display_data <= '0';
    else
     red_display_data <= '1';
     green_display_data <='0';
     blue_display_data <= '0';
    end if;
else

-- switch 1 display
If row_address(6 Downto 1) = "111010" and
   col_address(6 Downto 1) = "011000" Then
     display_item <= '1';
    If switch1 = '0' Then
     red_display_data <= '0';
     green_display_data <='1';
     blue_display_data <= '0';
    else
     red_display_data <= '1';
     green_display_data <='0';
     blue_display_data <= '0';
    end if;
else

-- sensor display
-- sensor 5
If row_address(6 Downto 1) = "011100" and
   col_address(6 Downto 1) = "100100" Then
      display_item <= '1';
   If sensor5 = '0' Then
     red_display_data <= '0';
     green_display_data <='1';
     blue_display_data <= '0';
    else
     red_display_data <= '1';
     green_display_data <='0';
     blue_display_data <= '0';
    end if;
else

-- sensor 1
If row_address(6 Downto 1) = "110000" and
  col_address(6 Downto 1) = "000100" Then
    display_item <= '1';
    If sensor1 = '0' Then
     red_display_data <= '0';
     green_display_data <='1';
     blue_display_data <= '0';
    else
     red_display_data <= '1';
     green_display_data <='0';
     blue_display_data <= '0';
    end if;
   else

-- sensor 2
If row_address(6 Downto 1) = "110000" and
   col_address(6 Downto 1) = "010100" Then
    display_item <= '1';
    If sensor2 = '0' Then
     red_display_data <= '0';
     green_display_data <='1';
     blue_display_data <= '0';
    else
     red_display_data <= '1';
     green_display_data <='0';
     blue_display_data <= '0';
    end if;
   else

-- sensor 3
If row_address(6 Downto 1) = "110000" and
   col_address(6 Downto 1) = "101100" Then
    display_item <= '1';
    If sensor3 = '0' Then
     red_display_data <= '0';
     green_display_data <='1';
     blue_display_data <= '0';
    else
     red_display_data <= '1';
     green_display_data <='0';
     blue_display_data <= '0';
    end if;
  else

-- sensor 4
If row_address(6 Downto 1) = "110000" and
   col_address(6 Downto 1) = "111011" Then
    display_item <= '1';
    If sensor4 = '0' Then
     red_display_data <= '0';
     green_display_data <='1';
     blue_display_data <= '0';
    else
     red_display_data <= '1';
     green_display_data <='0';
     blue_display_data <= '0';
    end if;
-- no items to display use memory for backgound colors
  else
   display_item <='0';
  end if;
end if;
end if;
end if;
end if;
end if;
end if;
end if;
end if;
end if;
end if;
end if;
end if;
end if;
End Process TRAIN_DISPLAY;

--Generate Horizontal and Vertical Timing Signals for Video Signal
VIDEO_DISPLAY: Process
Begin

Wait until(Clock'Event) and (Clock='1');
If Power_on = '1' Then
 H_count <= CONV_STD_LOGIC_VECTOR(0,10);
 V_count <= CONV_STD_LOGIC_VECTOR(0,10);
 Video_on_H <= '0';
 Video_on_V <= '0';
Else
-- H_count counts pixels (640 + extra time for sync signals)
--
--   <-Clock out RGB Pixel Row Data ->   <-H Sync->
--   ------------------------------------__________--------
--   0                           640   659       755    799
--
If (H_count >= H_max) then
   H_count <= To_Stdlogicvector(B"0000000000");
Else
   H_count <= H_count + To_Stdlogicvector(B"0000000001");
End if;

--Generate Horizontal Sync Signal
If (H_count <= CONV_STD_LOGIC_VECTOR(755,10)) and (H_count >= CONV_STD_LOGIC_VECTOR(659,10)) Then
   Horiz_Sync <= '0';
ELSE
   Horiz_Sync <= '1';
End if;

--V_count counts rows of pixels (480 + extra time for sync signals)
--
--  <---- 480 Horizontal Syncs (pixel rows) -->  ->V Sync<-
--  -----------------------------------------------_______------------
--  0                                       480    493-494          524
--
If (V_count >= V_max) and (H_count >= CONV_STD_LOGIC_VECTOR(699,10)) then
   V_count <= To_Stdlogicvector(B"0000000000");
Else If (H_count = CONV_STD_LOGIC_VECTOR(699,10)) Then
   V_count <= V_count + To_Stdlogicvector(B"0000000001");
End if;
End if;

-- Generate Vertical Sync Signal
If (V_count <= CONV_STD_LOGIC_VECTOR(494,10)) and (V_count >= CONV_STD_LOGIC_VECTOR(493,10)) Then
   Vert_Sync <= '0';
   Debounce_CLock <= '0';
ELSE
   Vert_Sync <= '1';
   Debounce_Clock <= '1';
End if;



-- Generate Video on Screen Signals for Pixel Data
-- Generate row and col address for 5 by 4 superpixel to map into 64 by 64 video memory
--
If (H_count <= CONV_STD_LOGIC_VECTOR(639,10)) Then
   video_on_H <= '1';
If pixel_col_count < CONV_STD_LOGIC_VECTOR(4,6) Then
   pixel_col_count <= pixel_col_count + '1';
Else
   pixel_col_count <= "0000000";
   col_address <= col_address + '1';
End if;
ELSE
   video_on_H <= '0';
   pixel_col_count <= "000000";
   col_address <= "000000";
End if;

IF(H_COUNT = CONV_STD_LOGIC_VECTOR(641,10)) Then
    pixel_row_count <= pixel_row_count + '1';
If (pixel_row_count = CONV_STD_LOGIC_VECTOR(3,6)) THEN
   pixel_row_count <= "000000";
   row_address <= row_address + '1';
End if;
End if;

If (V_count <= CONV_STD_LOGIC_VECTOR(479,10)) Then
   video_on_V <= '1';
ELSE
   video_on_V <= '0';
   pixel_row_count <= "000000";
   row_address <= "000000";
End if;


If (V_count = CONV_STD_LOGIC_VECTOR(0,10)) and (H_count = CONV_STD_LOGIC_VECTOR(0,10)) then
If (F_count = CONV_STD_LOGIC_VECTOR(30,5)) then
   F_count <= To_Stdlogicvector(B"00000");
   second <= not second;
Else 
   F_count <= F_count + To_Stdlogicvector(B"00001");
End if;
End if;
End if;


end process VIDEO_DISPLAY;

-- Connections to LED Display Segments
MSB_dp <= PB2_DEBOUNCED_SYNC;
LSB_dp <= PB1_DEBOUNCED_SYNC;
MSB <= Track4 & Track3 & Track2 & Track1;
LSB <= DirA & DirB;

MSB_a <= NOT MSB_7SEG(6);
MSB_b <= NOT MSB_7SEG(5);
MSB_c <= NOT MSB_7SEG(4);
MSB_d <= NOT MSB_7SEG(3);
MSB_e <= NOT MSB_7SEG(2);
MSB_f <= NOT MSB_7SEG(1);
MSB_g <= NOT MSB_7SEG(0);

LSB_a <= NOT LSB_7SEG(6);
LSB_b <= NOT LSB_7SEG(5);
LSB_c <= NOT LSB_7SEG(4);
LSB_d <= NOT LSB_7SEG(3);
LSB_e <= NOT LSB_7SEG(2);
LSB_f <= NOT LSB_7SEG(1);
LSB_g <= NOT LSB_7SEG(0);

LED_DISPLAY: process  (MSB,LSB)
-- BCD to 7 Segment Decoders for LED Displays
begin
CASE MSB IS
        WHEN "0000" =>
            MSB_7SEG <= "1111110";
        WHEN "0001" =>
            MSB_7SEG <= To_StdLogicVector(B"0110000");
        WHEN "0010" =>
            MSB_7SEG <= To_StdLogicVector(B"1101101");
        WHEN "0011" =>
            MSB_7SEG <= To_StdLogicVector(B"1111001");
        WHEN "0100" =>
            MSB_7SEG <= To_StdLogicVector(B"0110011");
        WHEN "0101" =>
            MSB_7SEG <= To_StdLogicVector(B"1011011");
        WHEN "0110" =>
            MSB_7SEG <= To_StdLogicVector(B"1011111");
        WHEN "0111" =>
            MSB_7SEG <= To_StdLogicVector(B"1110000");
        WHEN "1000" =>
            MSB_7SEG <= To_StdLogicVector(B"1111111");
        WHEN "1001" =>
            MSB_7SEG <= To_StdLogicVector(B"1111011"); 
        WHEN "1010" =>
            MSB_7SEG <= To_StdLogicVector(B"1110111");
        WHEN "1011" =>
            MSB_7SEG <= To_StdLogicVector(B"0011111");
        WHEN "1100" =>
            MSB_7SEG <= To_StdLogicVector(B"1001110");
        WHEN "1101" =>
            MSB_7SEG <= To_StdLogicVector(B"0111101"); 
        WHEN "1110" =>
            MSB_7SEG <= To_StdLogicVector(B"1001111");
        WHEN "1111" =>
            MSB_7SEG <= To_StdLogicVector(B"1000111");
	WHEN OTHERS =>
            MSB_7SEG <= To_StdLogicVector(B"0000001");
END CASE;

CASE LSB IS
        WHEN "0000" =>
            LSB_7SEG <= To_StdLogicVector(B"1111110");
        WHEN "0001" =>
            LSB_7SEG <= To_StdLogicVector(B"0110000");
        WHEN "0010" =>
            LSB_7SEG <= To_StdLogicVector(B"1101101");
        WHEN "0011" =>
            LSB_7SEG <= To_StdLogicVector(B"1111001");
        WHEN "0100" =>
            LSB_7SEG <= To_StdLogicVector(B"0110011");
        WHEN "0101" =>
            LSB_7SEG <= To_StdLogicVector(B"1011011");
        WHEN "0110" =>
            LSB_7SEG <= To_StdLogicVector(B"1011111");
        WHEN "0111" =>
            LSB_7SEG <= To_StdLogicVector(B"1110000");
        WHEN "1000" =>
            LSB_7SEG <= To_StdLogicVector(B"1111111");
        WHEN "1001" =>
            LSB_7SEG <= To_StdLogicVector(B"1111011"); 
        WHEN "1010" =>
            LSB_7SEG <= To_StdLogicVector(B"1110111");
        WHEN "1011" =>
            LSB_7SEG <= To_StdLogicVector(B"0011111");
        WHEN "1100" =>
            LSB_7SEG <= To_StdLogicVector(B"1001110");
        WHEN "1101" =>
            LSB_7SEG <= To_StdLogicVector(B"0111101"); 
        WHEN "1110" =>
            LSB_7SEG <= To_StdLogicVector(B"1001111");
        WHEN "1111" =>
            LSB_7SEG <= To_StdLogicVector(B"1000111");
	WHEN OTHERS =>
            LSB_7SEG <= To_StdLogicVector(B"0000001");
END CASE;

end process LED_DISPLAY;


-- Sync extenal pushbutton inputs to chip clock
PUSH_BUTTON: process (clock)
begin
  wait until (clock'event) and (clock='1');
PB1_Sync <= NOT PB1;
PB2_Sync <= NOT PB2;
Switch_Sync <= Switch;
PB1_DEBOUNCED_SYNC <= PB1_DEBOUNCED;
PB2_DEBOUNCED_SYNC <= PB2_DEBOUNCED;
end process PUSH_BUTTON;

-- Debounce Button: Filters out mechanical bounce for around 80Ms.
-- Debounce clock uses Vert_Sync timing signal (16Ms) to save hardware
-- for clock prescaler
DEBOUNCE_BUTTON1: process (debounce_clock)
begin
  wait until (debounce_clock'event) and (debounce_clock='1');
  SHIFT_PB1(2 Downto 0) <= SHIFT_PB1(3 Downto 1);
  SHIFT_PB1(3) <= PB1_Sync;
  If SHIFT_PB1(3 Downto 0)="1111" THEN
    PB1_DEBOUNCED <= '1';
  ELSE 
    PB1_DEBOUNCED <= '0';
  End if;
end process DEBOUNCE_BUTTON1;

DEBOUNCE_BUTTON2: process (debounce_clock)
begin
  wait until (debounce_clock'event) and (debounce_clock='1');
  SHIFT_PB2(2 Downto 0) <= SHIFT_PB2(3 Downto 1);
-- use switch 0 since PB2 broken on my board
  SHIFT_PB2(3) <= PB2_Sync;
  If SHIFT_PB2(3 Downto 0)="1111" THEN
    PB2_DEBOUNCED <= '1';
  ELSE 
    PB2_DEBOUNCED <= '0';
  End if;
end process DEBOUNCE_BUTTON2;

SINGLE_PULSE_PB1: process (Clock)
begin
  wait until (CLOCK'event) and (CLOCK='1');
  If RESET='1' Then
  PB1_SINGLE_PULSE <='0';
  PB1_DEBOUNCED_DELAY <= '1';
  ELSE
-- Generates Single Clock Cycle Pulse When Switch Hit
-- No matter how long switch is held down
  IF PB1_DEBOUNCED_SYNC = '1' AND PB1_DEBOUNCED_DELAY = '0' THEN
   PB1_SINGLE_PULSE <= '1';
  ELSE
   PB1_SINGLE_PULSE <= '0';
  END IF;
  PB1_DEBOUNCED_DELAY <= PB1_DEBOUNCED_SYNC;
 End if;
end process SINGLE_PULSE_PB1;

SINGLE_PULSE_PB2: process (Clock)
begin
  wait until (CLOCK'event) and (CLOCK='1');
   If RESET='1' Then
    PB2_SINGLE_PULSE <='0';
    PB2_DEBOUNCED_DELAY <= '1';
   ELSE
    PB2_DEBOUNCED_DELAY <= PB2_DEBOUNCED_SYNC;
-- Generates Single Clock Cycle Pulse When Switch Hit
-- No matter how long switch is held down
    IF PB2_DEBOUNCED_SYNC = '1' AND PB2_DEBOUNCED_DELAY = '0' THEN
     PB2_SINGLE_PULSE <= '1';
    ELSE
     PB2_SINGLE_PULSE <= '0';
    END IF;
  End if;
end process SINGLE_PULSE_PB2;


end behavior;

